import argparse
import json
from os import listdir
import os
import numpy as np
from ProcessedDatasetEvaluator import ProcessedDatasetEvaluator
from summary_loader import load_processed_dataset, load_tvsum_mat
from knapsack_implementation import knapSack

PROCESSED_SUMME = '../../data/SumMe/processed/eccv16_dataset_summe_google_pool5.h5'
PROCESSED_TVSUM = '../../data/TVSUM/processed/eccv16_dataset_tvsum_google_pool5.h5'


def upsample_summary(summary_scores, n_frames, positions, change_points):
    # compute the importance scores for the original length frame sequence
    frame_scores = np.zeros((n_frames), dtype=np.float32)
    if positions.dtype != int:
        positions = positions.astype(np.int32)
    if positions[-1] != n_frames:
        positions = np.concatenate([positions, [n_frames]])

    # fill the original length summary with the values of generated summary
    for i in range(len(positions) - 1):
        pos_left, pos_right = positions[i], positions[i + 1]
        if i == len(summary_scores):
            frame_scores[pos_left:pos_right] = 0
        else:
            frame_scores[pos_left:pos_right] = summary_scores[i]

    # summary = generate_summary(frame_scores, change_points)

    return frame_scores


def generate_summary(frame_scores, change_points):
    # Compute shot-level importance scores by taking the average importance scores of all frames in the shot
    shot_imp_scores = []
    shot_lengths = []
    for shot in change_points:
        shot_lengths.append(shot[1] - shot[0] + 1)
        shot_imp_scores.append((frame_scores[shot[0]:shot[1] + 1].mean()).item())

    # Select the best shots using the knapsack implementation
    final_max_length = int((shot[1] + 1) * 0.15)

    selected = knapSack(final_max_length, shot_lengths, shot_imp_scores, len(shot_lengths))

    # Select all frames from each selected shot (by setting their value in the summary vector to 1)
    summary = np.zeros(shot[1] + 1, dtype=np.int8)
    for shot in selected:
        summary[change_points[shot][0]:change_points[shot][1] + 1] = 1

    return summary


def average_results(video_scores):
    mean_scores = dict()
    for key in video_scores.keys():
        mean_scores[key] = np.asarray(video_scores[key]).mean(axis=0)

    return mean_scores


def resolve_video_names(json_file):
    with open(json_file) as f:
        data = json.loads(f.read())
        return list(data.keys())


def init_scores_dict(keys):
    all_scores = dict()
    all_scores = {el: [] for el in keys}  # init dict
    return all_scores


def evaluate_epoch_results(path, processed_dataset, metric):
    results = listdir(path)
    results.sort(key=lambda video: int(video[6:-5]))
    epoch0 = path + "/" + results[0]
    video_names = resolve_video_names(epoch0)
    video_scores = init_scores_dict(video_names)

    for idx, epoch in enumerate(results):
        with open(path + "/" + epoch) as f:
            data = json.loads(f.read())
            for video_name in video_names:
                n_frames = np.array(processed_dataset[video_name]['nframes'])
                positions = np.array(processed_dataset[video_name]['positions'])
                change_points = np.array(processed_dataset[video_name]['change_points'])
                scores = np.array(data[video_name])
                # upsample the generated summary to equal the original
                upsampled_summary = upsample_summary(scores, n_frames, positions, change_points)
                evaluator = ProcessedDatasetEvaluator(upsampled_summary, processed_dataset, metric)
                ref_summaries = np.array(processed_dataset[video_name]['user_score'])
                # evaluate
                video_scores[video_name].append(
                    evaluator.evaluate(upsampled_summary, ref_summaries))
    return video_scores


def arg_parser():
    results = '../../results/SumMe/seed31_non_overlapping'
    #results = '../../results/SumMe/original_splits/run3'

    # results = '../../results/TVSum/non-overlapping/with_random_seed/run1'
    # results = '../../results/sum-gan-sl/SumMe/original-splits'
    #results = '../../results/sum-gan-sl/SumMe/non-overlapping-splits'

    # results = '../../results/SumMe/original_splits/run3'

    parser = argparse.ArgumentParser(description="Generate Summary")
    parser.add_argument("--results_path", type=str, default=results,
                        help="Scores path from a split")
    parser.add_argument("--dataset", type=str, default='summe',
                        help="two datasets are available: summe and tvsum")
    parser.add_argument("--metric", type=str, default='spearmanr',
                        help="metric spearmanr or kendalltau ")
    return parser


if __name__ == "__main__":
    parser = arg_parser()
    args = parser.parse_args()
    metric = args.metric
    results_path = args.results_path
    type = args.dataset

    # load dataset
    if type == 'summe':
        dataset = load_processed_dataset(PROCESSED_SUMME, type)
    else:
        dataset = load_processed_dataset(PROCESSED_TVSUM, type)

    n_splits = 5
    split_eval = []
    for idx in range(n_splits):
        split = os.path.join(results_path, 'split{}'.format(idx))
        video_scores = evaluate_epoch_results(split, dataset, metric)
        # average the results over the total number of epochs
        # averaged = average_results(video_scores)
        split_eval.append(video_scores)
    print(split_eval)
    res = []
    for el in split_eval:
        res.append(list(el.values()))
    print(np.round(np.asarray(res).mean(), decimals=4))
